SystemOrganization addCategory: #'Zinc-REST-Server'!
SystemOrganization addCategory: #'Zinc-REST-Tests'!

TestCase subclass: #ZnExampleStorageRestServerDelegateTest
	instanceVariableNames: 'server client'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Tests'!

!ZnExampleStorageRestServerDelegateTest commentStamp: '<historical>' prior: 0!
ZnExampleStorageRestServerDelegateTest contains unit tests for the ZnExampleStorageRestServerDelegate in combination with the calls under ZnExampleStorageRestCall.
!

!ZnExampleStorageRestServerDelegateTest methodsFor: 'running' stamp: 'SvenVanCaekenberghe 12/10/2012 20:03'!
setUp
	server := ZnServer startOn: 1700 + 10 atRandom.
	server delegate: ZnExampleStorageRestServerDelegate new.
	client := ZnClient new
		url: server localUrl;
		addPathSegment: #storage;
		accept: ZnMimeType applicationJson;
		contentReader: [ :entity | entity ifNotNil: [ NeoJSONReader fromString: entity contents ] ];
		contentWriter: [ :object | ZnEntity with: (NeoJSONWriter toString: object) type: ZnMimeType applicationJson ];
		yourself! !

!ZnExampleStorageRestServerDelegateTest methodsFor: 'running' stamp: 'SvenVanCaekenberghe 12/10/2012 19:18'!
tearDown
	client close.
	server stop! !

!ZnExampleStorageRestServerDelegateTest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/12/2012 10:26'!
testCreate
	"You create a new entry by posting a JSON map representation to /storage/objects
	The result is a 201 Created response with the URI of the new object both
	returned as a JSON string and as the Location header"
	
	| data objectUri |
	data := Dictionary with: #x -> 'foo' with: #y -> 'bar'.
	client
		addPathSegment: 'objects';
		contents: data;
		post.
	self assert: client isSuccess.
	self assert: client isCreated.
	objectUri := client response location.
	self assert: objectUri isString & objectUri isEmpty not.
	self assert: client contents equals: objectUri.
	client resetEntity.
	client get.
	self assert: client isSuccess.
	self assert: (client contents includes: objectUri).
	client 
		url: objectUri; 
		get.
	self assert: client isSuccess.
	data keysAndValuesDo: [ :key :value |
		(client contents at: key) = value ].
	self assert: (client contents at: #'object-uri') equals: objectUri ! !

!ZnExampleStorageRestServerDelegateTest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/12/2012 10:26'!
testDelete
	"You delete an existing entry by doing a DELETE on its uri"
	
	| data objectUri |
	data := Dictionary with: #x -> 'foo' with: #y -> 'bar'.
	client
		addPathSegment: 'objects';
		contents: data;
		post.
	self assert: client isCreated.
	objectUri := client response location.
	client resetEntity.
	client 
		url: objectUri; 
		get.
	self assert: client isSuccess.
	client delete.
	self assert: client isSuccess.
	client get.
	self deny: client isSuccess.
	self assert: client isNotFound! !

!ZnExampleStorageRestServerDelegateTest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/10/2012 21:25'!
testEmpty
	"An empty database returns a empty array for GET /storage/objects"
	
	client
		addPathSegment: 'objects';
		get.
	self assert: client isSuccess.
	self assert: client contents equals: #()! !

!ZnExampleStorageRestServerDelegateTest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/12/2012 10:27'!
testQuery
	"Any query parameters are seen as a conjunctive filter.
	Here we execute a GET /storage/object?string=five to find a entry where
	the key 'string' equals 'five'. Note that the uri is returned."

	| data objectUri |
	client addPathSegment: 'objects'.
	1 to: 10 do: [ :each |
		data := Dictionary with: #int -> each with: #string -> each asWords.
		client
			contents: data;
			post.
		self assert: client isCreated ].
	client resetEntity.
	client
		queryAt: #string put: 'five';
		get.
	self assert: client isSuccess.
	objectUri := client contents first.	
	client
		url: objectUri; 
		get.
	self assert: client isSuccess.
	self assert: (client contents at: #int) equals: 5! !

!ZnExampleStorageRestServerDelegateTest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/12/2012 10:28'!
testUpdate
	"To update an existing object you PUT a new JSON map representation to an existing uri"
	
	| data objectUri |
	data := Dictionary with: #x -> 'foo' with: #y -> 'bar'.
	client
		addPathSegment: 'objects';
		contents: data;
		post.
	self assert: client isCreated.
	objectUri := client response location.
	client resetEntity.
	data at: #z put: 100.
	client
		url: objectUri; 
		contents: data;
		put.
	self assert: client isSuccess.
	client resetEntity.
	client get.
	self assert: client isSuccess.
	data keysAndValuesDo: [ :key :value |
		(client contents at: key) = value ].
	self assert: (client contents at: #'object-uri') equals: objectUri.
	self assert: (client contents at: #z) equals: 100! !

TestCase subclass: #ZnExampleSumRestCallTest
	instanceVariableNames: 'server client'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Tests'!

!ZnExampleSumRestCallTest commentStamp: '<historical>' prior: 0!
ZnExampleSumRestCallTest contains unit tests for ZnExampleRestCall.!

!ZnExampleSumRestCallTest methodsFor: 'running' stamp: 'SvenVanCaekenberghe 12/10/2012 17:28'!
setUp
	server := ZnServer startOn: 1700 + 10 atRandom.
	server delegate: 
		(ZnRestServerDelegate new 
			uriSpace: (ZnCallHierarchyRestUriSpace new 
							rootClass: ZnExampleSumRestCall;
							yourself); 
			yourself).
	client := ZnClient new
		url: server localUrl;
		addPathSegment: #sum;
		yourself! !

!ZnExampleSumRestCallTest methodsFor: 'running' stamp: 'SvenVanCaekenberghe 12/10/2012 17:28'!
tearDown
	client close.
	server stop! !

!ZnExampleSumRestCallTest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/10/2012 21:22'!
testPostSum
	"Here we test POSTing a text file with a number per line to /sum"
	
	| input output |
	input := #(1 2 3).
	client entity: (ZnEntity text: (String crlf join: input)).
	output := client post.
	self assert: client isSuccess.
	self assert: output equals: input sum asString! !

!ZnExampleSumRestCallTest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/11/2012 09:09'!
testQuerySum
	"Here we test a GET /sum?numbers=1,2,3"

	| input output |
	input := #(1 2 3).
	client queryAt: #numbers put: ($, join: input).
	output := client get.
	self assert: client isSuccess.
	self assert: output equals: input sum asString! !

!ZnExampleSumRestCallTest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/10/2012 21:21'!
testUriSum
	"Here we test doing a GET /sum/1/2/3"
	
	| input output |
	input := #(1 2 3).
	input do: [ :each | client addPathSegment: each asString ].
	output := client get.
	self assert: client isSuccess.
	self assert: output equals: input sum asString! !

Exception subclass: #ZnRestCallUnauthorized
	instanceVariableNames: 'call'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Server'!

!ZnRestCallUnauthorized class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 6/16/2011 15:17'!
call: call
	^ self new
		call: call;
		yourself! !

!ZnRestCallUnauthorized methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2011 15:17'!
call
	^ call! !

!ZnRestCallUnauthorized methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2011 15:17'!
call: anObject
	call := anObject! !

!ZnRestCallUnauthorized methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2011 15:18'!
request
	^ call request! !

Object subclass: #ZnRestCall
	instanceVariableNames: 'request response variables delegate principal credentials'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Server'!

!ZnRestCall commentStamp: 'SvenVanCaekenberghe 6/15/2011 16:34' prior: 0!
I am ZnRestCall.

I represent the actual execution of a REST call.
!

ZnRestCall subclass: #ZnAutoMatchedRestCall
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Server'!

!ZnAutoMatchedRestCall commentStamp: 'SvenVanCaekenberghe 6/15/2011 16:35' prior: 0!
I am ZnAutoMatchedRestCall.
I am a ZnRestCall.

My class side knows if I match a certain request.
A default implementation uses #pattern specification.!

!ZnAutoMatchedRestCall class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/9/2012 17:45'!
isAbstract
	^ self = ZnAutoMatchedRestCall ! !

!ZnAutoMatchedRestCall class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/9/2012 19:45'!
match: request
	self isAbstract ifTrue: [ ^ nil ].
	^ ZnRestServerUtils 
		matchPath: request uri pathSegments 
		pattern: self pattern   ! !

!ZnAutoMatchedRestCall class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/15/2011 21:47'!
pattern
	^ self subclassResponsibility ! !

ZnAutoMatchedRestCall subclass: #ZnExampleSumRestCall
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Tests'!

!ZnExampleSumRestCall commentStamp: '<historical>' prior: 0!
I am ZnExampleSumRestCall.
I am a ZnAutoMatchedRestCall.

I handle REST requests for /sum

For a GET request, I accept numbers as path elements or as a query parameter.

	GET /sum/1/2/3
	GET /sum?numbers=1,2,3
	
For a POST requests, I accept a text/plain entity containing numbers.

	POST /sum '1,2,3'
	
To install me, try:

	ZnServer startDefaultOn: 1701.
	ZnServer default logToTranscript.

	ZnServer default delegate
		map: 'sum' 
		to: (ZnRestServerDelegate new 
				uriSpace: (ZnCallHierarchyRestUriSpace new 
								rootClass: ZnExampleSumRestCall;
								yourself); 
				yourself).
			
	ZnServer stopDefault.
	
Here are some example invocations:

	ZnClient new
		url: ZnServer default localUrl;
		addPathSegment: #sum;
		queryAt: #numbers put: '1,2,3'; 
		get.
		
	ZnClient new
		url: ZnServer default localUrl;
		addPathSegment: #sum;
		addPathSegment: '1';
		addPathSegment: '2';
		addPathSegment: '3';
		get.
		
	ZnClient new
		url: ZnServer default localUrl;
		addPathSegment: #sum;
		entity: (ZnEntity text: '1,2,3'); 
		post.!

!ZnExampleSumRestCall class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/9/2012 17:47'!
match: request
	^ request uri firstPathSegment = 'sum'
		ifTrue: [ request uri pathSegments allButFirst ]
		ifFalse: [ nil ]    ! !

!ZnExampleSumRestCall methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/9/2012 19:17'!
get
	| sum |
	sum := self numbersForGet inject: 0 into: [ :sub :next | sub + next ].
	self response: (ZnResponse ok: (ZnEntity with: sum asString))! !

!ZnExampleSumRestCall methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/9/2012 19:16'!
numbersForGet
	variables isEmpty ifFalse: [ ^ variables collect: [ :each | 
			[ each asNumber ] on: Error do: [ 0 ] ] ].
	(self request uri queryAt: #numbers ifAbsent: [ nil ])
		ifNotNil: [ :numbers | 
			^ (numbers findTokens: { Character space. Character tab. $, }) collect: [ :each | 
				[ each asNumber ] on: Error do: [ 0 ] ] ].
	^ #()! !

!ZnExampleSumRestCall methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/9/2012 19:18'!
numbersForPost
	(self request hasEntity and: [ self request entity contentType = ZnMimeType textPlain])
		ifNotNil: [ :numbers | 
			^ (self request entity contents findTokens: { Character space. Character tab. Character cr. Character lf. $, }) collect: [ :each | 
				[ each asNumber ] on: Error do: [ 0 ] ] ].
	^ #()! !

!ZnExampleSumRestCall methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/9/2012 19:18'!
post
	| sum |
	sum := self numbersForPost inject: 0 into: [ :sub :next | sub + next ].
	self response: (ZnResponse ok: (ZnEntity with: sum asString))! !

ZnAutoMatchedRestCall subclass: #ZnJSONRestCall
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Server'!

!ZnJSONRestCall commentStamp: '<historical>' prior: 0!
I am ZnJSONRestCall.
I am a ZnAutoMatchedRestCall and a ZnRestCall.

I offer extra support for the common case where most responses or requests use JSON as representation.

I work together with ZnJSONRestServerDelegate.!

ZnJSONRestCall subclass: #ZnExampleStorageRestCall
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Tests'!

!ZnExampleStorageRestCall commentStamp: '<historical>' prior: 0!
I am ZnExampleStorageRestCall, the root of a hierarchy of REST calls implementing a simple storage service with CRUD operations.

The calls implemented by my subclasses are used by ZnExampleStorageRestServerDelegate.!

ZnExampleStorageRestCall subclass: #ZnExampleStorageObjectRestCall
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Tests'!

!ZnExampleStorageObjectRestCall commentStamp: '<historical>' prior: 0!
ZnExampleStorageObjectRestCall.

GET /storage/object/<id>

	return the map stored under id
	
DELETE /storage/object/<id>

	delete the map stored under id
	
PUT /storage/object/<id> <MAP>

	overwrite the map stored under id with MAP

	#'object-uri' is a reserved key in MAP!

!ZnExampleStorageObjectRestCall class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2012 16:03'!
pattern
	^ #( 'storage' 'objects' '*' )! !

!ZnExampleStorageObjectRestCall methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/10/2012 16:41'!
delete
	| objectUri |
	objectUri := '/storage/objects/{1}' format: { variables first }.
	self delegate storage removeKey: objectUri ifAbsent: [ ^ self notFound ].
	self emptyOkResponse! !

!ZnExampleStorageObjectRestCall methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/10/2012 16:39'!
get
	| data objectUri |
	objectUri := '/storage/objects/{1}' format: { variables first }.
	data := self delegate storage at: objectUri ifAbsent: [ ^ self notFound ].
	self jsonResponse: data! !

!ZnExampleStorageObjectRestCall methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/10/2012 16:47'!
put
	| data objectUri |
	objectUri := '/storage/objects/{1}' format: { variables first }.
	(self delegate storage includesKey: objectUri) ifFalse: [ ^ self notFound ].
	data := self jsonRequest.
	data at: #'object-uri' put: objectUri.
	self delegate storage at: objectUri put: data.
	self emptyOkResponse ! !

ZnExampleStorageRestCall subclass: #ZnExampleStorageObjectsRestCall
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Tests'!

!ZnExampleStorageObjectsRestCall commentStamp: '<historical>' prior: 0!
ZnExampleStorageObjectsRestCall.

GET /storage/objects

	list all stored maps as a list of object-uris
	
GET /storage/objects?key1=value1&key2=value2

	as above with conjunctive condition specified 
	
POST /storage/objects <MAP>
	
	store a new map, returns the created object-uri

	#'object-uri' is a reserved key in MAP!

!ZnExampleStorageObjectsRestCall class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2012 13:27'!
pattern
	^ #( 'storage' 'objects' )! !

!ZnExampleStorageObjectsRestCall methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/10/2012 15:58'!
get
	| predicateKeys |
	predicateKeys := request uri queryKeys.
	predicateKeys isEmpty
		ifTrue: [ self jsonResponse: self delegate storage keys ]
		ifFalse: [ 
			| objectKeys |
			objectKeys := OrderedCollection new.
			self delegate storage
				keysAndValuesDo: [ :key :value | 
					(predicateKeys allSatisfy: [ :each | (value at: each ifAbsent: [ nil ]) = (request uri queryAt: each) ])
						ifTrue: [ objectKeys add: key ] ].
			self jsonResponse: objectKeys ]! !

!ZnExampleStorageObjectsRestCall methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/10/2012 16:34'!
post
	| data newUri |
	data := self jsonRequest.
	data isDictionary
		ifTrue: [ 
			newUri := '/storage/objects/{1}' format: { self delegate nextId printString }.
			data at: #'object-uri' put: newUri.
			self delegate storage at: newUri put: data.
			self created: newUri ]
		ifFalse: [ self badRequest: 'JSON map expected' ]! !

!ZnExampleStorageRestCall class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2012 13:18'!
isAbstract
	^ self = ZnExampleStorageRestCall ! !

!ZnJSONRestCall class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2012 13:18'!
isAbstract
	^ self = ZnJSONRestCall ! !

!ZnJSONRestCall methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/10/2012 13:08'!
acceptMethods: allowedMethods
	"Allow and execute allowedMethods (#put or #delete) as POST ? method=put"
	
	| method |
	method := (request uri queryAt: 'method' ifAbsent: [ #error ]) asLowercase asSymbol.
	(allowedMethods includes: method)
		ifTrue: [
			self perform: method ]
		ifFalse: [ 
			self wrongMethod ]    	! !

!ZnJSONRestCall methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 12/10/2012 13:08'!
badRequest: message
	response := self delegate 
		errorResponse: request 
		code: 400 
		message: message! !

!ZnJSONRestCall methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 12/10/2012 13:09'!
callUnauthorized
	response := self delegate 
		errorResponse: request 
		code: 401
		message: 'Unauthorized during ', self class name! !

!ZnJSONRestCall methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/10/2012 16:15'!
created: uri
	^ (response := ZnResponse new)
		statusLine: ZnStatusLine created;
		headers: ZnHeaders defaultResponseHeaders;
		setLocation: uri;
		entity: (ZnEntity with: (NeoJSONWriter toString: uri) , String crlf type: ZnMimeType applicationJson);
		yourself! !

!ZnJSONRestCall methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/10/2012 13:08'!
emptyOkResponse 
	^ response := ZnResponse ok: (ZnEntity text: '')! !

!ZnJSONRestCall methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2012 13:10'!
jsonRequest
	^ self request contentType = ZnMimeType applicationJson
		ifTrue: [
			NeoJSONReader fromString: self request contents ]
		ifFalse: [
			self error: 'JSON content expected' ]! !

!ZnJSONRestCall methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2012 16:28'!
jsonResponse: data
	| json |
	json := String
		streamContents: [ :stream | 
			(NeoJSONWriter on: stream)
				prettyPrint: true;
				newLine: String crlf;
				nextPut: data.
			stream crlf ].
	self response: (ZnResponse ok: (ZnEntity with: json type: ZnMimeType applicationJson))! !

!ZnJSONRestCall methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 12/10/2012 13:09'!
notFound
	response := self delegate 
		errorResponse: request 
		code: 404 
		message: 'Resource not found during ', self class name! !

!ZnJSONRestCall methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2012 13:07'!
queryAt: key ifAbsent: block
	^ self request uri queryAt: key ifAbsent: block! !

!ZnJSONRestCall methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 12/10/2012 13:09'!
serverError: message
	response := self delegate 
		serverError: request 
		exception: message! !

!ZnJSONRestCall methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 12/10/2012 13:09'!
wrongMethod
	response := self delegate 
		errorResponse: request 
		code: 405
		message: 'Wrong method'! !

!ZnRestCall methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2011 14:31'!
credentials
	^ credentials! !

!ZnRestCall methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/16/2011 14:31'!
credentials: anObject
	credentials := anObject! !

!ZnRestCall methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/15/2011 10:58'!
delegate
	^ delegate! !

!ZnRestCall methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2011 10:58'!
delegate: anObject
	delegate := anObject! !

!ZnRestCall methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/15/2011 11:07'!
execute
	^ (self respondsTo: self method)	
		ifTrue: [ (self perform: self method) response ] 
		ifFalse: [ self delegate wrongMethod: request ]  ! !

!ZnRestCall methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/16/2011 15:22'!
isAuthorized
	^ true    ! !

!ZnRestCall methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/15/2011 10:56'!
method
	^ request method asLowercase asSymbol ! !

!ZnRestCall methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/15/2011 11:08'!
principal
	^ principal! !

!ZnRestCall methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2011 11:08'!
principal: anObject
	principal := anObject! !

!ZnRestCall methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/15/2011 10:58'!
request
	^ request! !

!ZnRestCall methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2011 10:58'!
request: anObject
	request := anObject! !

!ZnRestCall methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/15/2011 10:58'!
response
	^ response! !

!ZnRestCall methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/15/2011 10:58'!
response: anObject
	response := anObject! !

!ZnRestCall methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/15/2011 10:58'!
variables
	^ variables! !

!ZnRestCall methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2011 10:58'!
variables: anObject
	variables := anObject! !

Object subclass: #ZnRestServerDelegate
	instanceVariableNames: 'uriSpace server debug'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Server'!

!ZnRestServerDelegate commentStamp: '<historical>' prior: 0!
I am ZnRestServerDelegate.
I am a ZnServer delegate.

Based on my URI space I dispatch requests to REST call objects.!

ZnRestServerDelegate subclass: #ZnJSONRestServerDelegate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Server'!

!ZnJSONRestServerDelegate commentStamp: '<historical>' prior: 0!
I am ZnJSONRestServerDelegate.
I am a ZnRestServerDelegate.

I offer extra support for the common case where most responses or requests use JSON as representation.

I work together with ZnJSONRestCall.!

ZnJSONRestServerDelegate subclass: #ZnExampleStorageRestServerDelegate
	instanceVariableNames: 'storage idCounter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Tests'!

!ZnExampleStorageRestServerDelegate commentStamp: '<historical>' prior: 0!
I am ZnExampleStorageRestServerDelegate.

I offer a REST interface on /storage with CRUD operations on JSON maps. I automatically use the call hierarchy below ZnExampleStorageRestCall.

To install me you can do:

	ZnServer startDefaultOn: 1701.
	ZnServer default logToTranscript.

	ZnServer default delegate
		map: 'storage' 
		to: ZnExampleStorageRestServerDelegate new.
			
	ZnServer stopDefault.

See ZnExampleStorageRestServerDelegateTests for example client usage.
!

!ZnExampleStorageRestServerDelegate methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/10/2012 15:47'!
initialize 
	idCounter := 1000.
	self uriSpace: ZnCallHierarchyRestUriSpace new.
	self uriSpace rootClass: ZnExampleStorageRestCall! !

!ZnExampleStorageRestServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2012 15:47'!
nextId
	^ idCounter := idCounter + 1! !

!ZnExampleStorageRestServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2012 13:13'!
storage
	^ storage ifNil: [ storage := Dictionary new ]! !

!ZnJSONRestServerDelegate methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 12/10/2012 16:30'!
errorResponse: request code: code message: message
	| statusLine data json |
	statusLine := ZnStatusLine code: code.
	(data := Dictionary new)
		at: 'method' put: request method;
		at: 'uri' put: request uri printString;
		at: 'status-code' put: statusLine code;
		at: 'status-message' put: statusLine reason;
		at: 'message' put: message.
	json := String
		streamContents: [ :stream | 
			(NeoJSONWriter on: stream)
				prettyPrint: true;
				newLine: String crlf;
				nextPut: data.
			stream crlf ].
	^ ZnResponse new
		statusLine: statusLine;
		headers: ZnHeaders defaultResponseHeaders;
		entity: (ZnEntity with: json type: ZnMimeType applicationJson);
		yourself! !

!ZnRestServerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/16/2011 15:22'!
authenticate: call
	| authorization |
	authorization := ZnRestServerUtils extractBasicAuthorization: call request.
	^ (self isAuthorizationValid: authorization) 
		ifTrue: [
			authorization ifNotNil: [ 
				call 
					principal: authorization key; 
					credentials: authorization value ].
			call isAuthorized ] 
		ifFalse: [ false ]   ! !

!ZnRestServerDelegate methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 6/16/2011 21:10'!
callUnauthorized: request
	^ self 
		errorResponse: request 
		code: 401 
		message: 'Call unauthorized'! !

!ZnRestServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/15/2011 10:58'!
debug
	^ debug ifNil: [ debug := false ]! !

!ZnRestServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/15/2011 10:57'!
debug: anObject
	debug := anObject! !

!ZnRestServerDelegate methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 6/16/2011 14:01'!
errorResponse: request code: code message: message
	| statusLine text |
	statusLine := ZnStatusLine code: code.
	text := String streamContents: [ :stream | 
		stream << message; space; nextPut: $[; print: code; space; << statusLine reason; nextPut: $]; crlf ].
	^ ZnResponse new
		statusLine: statusLine;
		headers: ZnHeaders defaultResponseHeaders;
		entity: (ZnEntity with: text);
		yourself! !

!ZnRestServerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/16/2011 15:19'!
execute: call
	^ [ call execute ]
		on: ZnRestCallUnauthorized 
		do: [ :exception | self unauthorized: exception request ]! !

!ZnRestServerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/17/2011 15:36'!
handleRequest: request
	| call |
	(call := self match: request)
		ifNil: [ ^ self noHandlerFound: request ].
	(self authenticate: call)
		ifFalse: [ ^ self callUnauthorized: request ].
	^ self debug 
		ifTrue: [ self execute: call ]
		ifFalse: [ 
			[ self execute: call ] 
				on: Exception 
				do: [ :exception | self serverError: request exception: exception ] ]! !

!ZnRestServerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/16/2011 15:20'!
isAuthorizationValid: authorization
	^ authorization isNil
		or: [ authorization key isEmptyOrNil not 
			and: [ authorization value isEmptyOrNil not ] ]! !

!ZnRestServerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/15/2011 17:34'!
match: request
	| call |
	(call := uriSpace match: request)
		ifNotNil: [
			call
				request: request; 
				delegate: self ].
	^ call
	! !

!ZnRestServerDelegate methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 6/16/2011 21:10'!
noHandlerFound: request
	^ self 
		errorResponse: request 
		code: 404 
		message: 'No handler found'! !

!ZnRestServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/15/2011 10:57'!
server
	^ server! !

!ZnRestServerDelegate methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2011 10:57'!
server: anObject
	server := anObject! !

!ZnRestServerDelegate methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 6/20/2011 17:34'!
serverError: request exception: exception
	^ self 
		errorResponse: request 
		code: 500 
		message: exception asString! !

!ZnRestServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/15/2011 10:57'!
uriSpace
	^ uriSpace! !

!ZnRestServerDelegate methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2011 10:57'!
uriSpace: anObject
	uriSpace := anObject! !

!ZnRestServerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/9/2012 18:07'!
value: request
	"I implement the generic #value: message as equivalent to #handleRequest:"
	
	^ self handleRequest: request! !

!ZnRestServerDelegate methodsFor: 'error handling' stamp: 'SvenVanCaekenberghe 6/16/2011 21:11'!
wrongMethod: request
	^ self 
		errorResponse: request 
		code: 405 
		message: 'Wrong HTTP method'! !

Object subclass: #ZnRestServerUtils
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Server'!

!ZnRestServerUtils commentStamp: 'SvenVanCaekenberghe 6/15/2011 16:19' prior: 0!
I am ZnRestServerUtils.!

!ZnRestServerUtils class methodsFor: 'utils' stamp: 'SvenVanCaekenberghe 6/15/2011 16:18'!
extractBasicAuthorization: request
	| authorization |
	authorization := (request headers at: 'Authorization' ifAbsent: [ ^ nil ]) findTokens: ' '.
	(authorization size = 2 and: [ authorization first = 'Basic' ]) ifFalse: [ ^ nil ].
	authorization :=  (ZnUtils decodeBase64: authorization second) findTokens: ':'.
	authorization size = 2 ifFalse: [ ^ nil ].
	^ authorization first -> authorization second
! !

!ZnRestServerUtils class methodsFor: 'utils' stamp: 'SvenVanCaekenberghe 6/16/2011 14:17'!
matchPath: path pattern: pattern
	"self matchPath: #('foo' '1' 'bar' '2') pattern: #('foo' '*' 'bar' '*')"
	
	| result |
	path isNil ifTrue: [ ^ pattern isNil ifTrue: [ #() ] ifFalse: [ nil ] ].
	(pattern isNil or: [ path size ~= pattern size ]) ifTrue: [ ^ nil ].
	(pattern includes: '*') 
		ifTrue: [ result := OrderedCollection new ]
		ifFalse: [ result := true ].
	pattern with: path do: [ :target :source |
		target = '*'
			ifTrue: [ 
				result add: source ]
			ifFalse: [ 
				source = target ifFalse: [ ^ nil ] ] ].
	^ result! !

Object subclass: #ZnRestUriSpace
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Server'!

!ZnRestUriSpace commentStamp: 'SvenVanCaekenberghe 6/15/2011 16:15' prior: 0!
I am ZnRestUriSpace.

I can #match: a request to a REST call.!

ZnRestUriSpace subclass: #ZnCallHierarchyRestUriSpace
	instanceVariableNames: 'rootClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-REST-Server'!

!ZnCallHierarchyRestUriSpace commentStamp: 'SvenVanCaekenberghe 6/15/2011 16:16' prior: 0!
I am ZnCallHierarchyRestUriSpace.
I am a ZnRestUriSpace.

I represent a class hierarchy of REST call subclasses, below my root class.!

!ZnCallHierarchyRestUriSpace methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/9/2012 17:43'!
match: request
	rootClass withAllSubclassesDo: [ :each |
		(each match: request)
			ifNotNil: [ :variables |
				^ each new
					variables: variables;
					yourself ] ].
	^ nil  ! !

!ZnCallHierarchyRestUriSpace methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/15/2011 10:57'!
rootClass
	^ rootClass! !

!ZnCallHierarchyRestUriSpace methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2011 10:57'!
rootClass: anObject
	rootClass := anObject! !

!ZnRestUriSpace methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/15/2011 17:30'!
match: request
	self subclassResponsibility ! !
